@@ -9,13 +9,16 @@ from .move import *
 from .product import *
 from .inventory import *
 from .configuration import *
+from .party import PartyReplace
 
 
 def register():
     Pool.register(
         Location,
         Party,
-        ProductsByLocationsStart,
+        PartyLocation,
+        ProductsByLocationsContext,
+        LocationLeadTime,
         Move,
         ShipmentIn,
         ShipmentInReturn,
@@ -30,21 +33,21 @@ def register():
         Cache,
         Template,
         Product,
-        ProductByLocationStart,
+        ProductByLocationContext,
         ProductQuantitiesByWarehouse,
-        ProductQuantitiesByWarehouseStart,
+        ProductQuantitiesByWarehouseContext,
         Inventory,
         InventoryLine,
         Configuration,
+        ConfigurationSequence,
+        ConfigurationLocation,
         module='stock', type_='model')
     Pool.register(
-        ProductsByLocations,
         AssignShipmentOut,
         AssignShipmentInternal,
         AssignShipmentInReturn,
-        ProductByLocation,
-        OpenProductQuantitiesByWarehouse,
         RecomputeCostPrice,
+        PartyReplace,
         module='stock', type_='wizard')
     Pool.register(
         DeliveryNote,
