@@ -1,15 +1,15 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
+import hashlib
 import operator
 from decimal import Decimal
-from functools import partial
 from collections import OrderedDict, defaultdict
 from itertools import groupby
 
 from sql import Literal, Union, Column, Null, For
 from sql.aggregate import Sum
-from sql.conditionals import Coalesce
+from sql.conditionals import Coalesce, Case
 from sql.operators import Concat
 
 from trytond.model import Workflow, Model, ModelView, ModelSQL, fields, Check
@@ -19,7 +19,7 @@ from trytond.tools import reduce_ids
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
-from trytond.modules.product import price_digits, TemplateFunction
+from trytond.modules.product import price_digits
 
 __all__ = ['StockMixin', 'Move']
 
@@ -31,6 +31,9 @@ LOCATION_DOMAIN = [
     If(Eval('state').in_(['staging', 'draft', 'cancel']),
         ('type', 'not in', ['warehouse']),
         ('type', 'not in', ['warehouse', 'view'])),
+    If(~Eval('state').in_(['done', 'cancel']),
+        ('active', '=', True),
+        ()),
     ]
 LOCATION_DEPENDS = ['state']
 
@@ -154,8 +157,13 @@ class Move(Workflow, ModelSQL, ModelView):
     _order_name = 'product'
     product = fields.Many2One("product.product", "Product", required=True,
         select=True, states=STATES,
-        domain=[('type', '!=', 'service')],
-        depends=DEPENDS)
+        domain=[
+            ('type', '!=', 'service'),
+            If(Bool(Eval('product_uom_category')),
+                ('default_uom_category', '=', Eval('product_uom_category')),
+                ())
+            ],
+        depends=DEPENDS + ['product_uom_category'])
     product_uom_category = fields.Function(
         fields.Many2One('product.uom.category', 'Product Uom Category'),
         'on_change_with_product_uom_category')
@@ -189,11 +197,13 @@ class Move(Workflow, ModelSQL, ModelView):
                 | Eval('shipment'))
             }, depends=['state', 'shipment'],
         select=True)
-    effective_date = fields.Date("Effective Date", readonly=True, select=True,
+    effective_date = fields.Date("Effective Date", select=True,
         states={
             'required': Eval('state') == 'done',
+            'readonly': (Eval('state').in_(['cancel', 'done'])
+                | Eval('shipment')),
             },
-        depends=['state'])
+        depends=['state', 'shipment'])
     state = fields.Selection([
         ('staging', 'Staging'),
         ('draft', 'Draft'),
@@ -260,18 +270,19 @@ class Move(Workflow, ModelSQL, ModelView):
             'set_state_assigned': ('You can not set stock move "%s" to '
                 'assigned state.'),
             'set_state_done': 'You can not set stock move "%s" to done state.',
-            'del_draft_cancel': ('You can not delete stock move "%s" because '
-                'it is not in draft or cancelled state.'),
+            'delele_state': ('You can not delete stock move "%s" because '
+                'it is not in staging, draft nor cancelled state.'),
             'period_closed': ('You can not modify move "%(move)s" because '
                 'period "%(period)s" is closed.'),
             'modify_assigned': ('You can not modify stock move "%s" because '
                 'it is in "Assigned" state.'),
             'modify_done_cancel': ('You can not modify stock move "%s" '
                 'because it is in "Done" or "Cancel" state.'),
-            'no_origin': 'The stock move "%s" has no origin.',
+            'no_origin': 'The stock moves "%s" have no origin.',
             })
         cls._transitions |= set((
                 ('staging', 'draft'),
+                ('staging', 'cancel'),
                 ('draft', 'assigned'),
                 ('draft', 'done'),
                 ('draft', 'cancel'),
@@ -447,22 +458,23 @@ class Move(Workflow, ModelSQL, ModelView):
 
     @fields.depends('from_location', 'to_location')
     def on_change_with_unit_price_required(self, name=None):
-        if (self.from_location
-                and self.from_location.type in ('supplier', 'production')):
+        from_type = self.from_location.type if self.from_location else None
+        to_type = self.to_location.type if self.to_location else None
+
+        if from_type == 'supplier' and to_type == 'storage':
             return True
-        if (self.to_location
-                and self.to_location.type == 'customer'):
+        if from_type == 'production':
             return True
-        if (self.from_location and self.to_location
-                and self.from_location.type == 'storage'
-                and self.to_location.type == 'supplier'):
+        if from_type == 'storage' and to_type == 'customer':
+            return True
+        if from_type == 'storage' and to_type == 'supplier':
             return True
         return False
 
     @fields.depends('from_location')
     def on_change_with_assignation_required(self, name=None):
         if self.from_location:
-            return self.from_location.type == 'storage'
+            return self.from_location.type in {'storage', 'view'}
 
     @staticmethod
     def _get_shipment():
@@ -527,9 +539,9 @@ class Move(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return [('product',) + tuple(clause[1:])]
+        return [('product.rec_name',) + tuple(clause[1:])]
 
-    def _update_product_cost_price(self, direction):
+    def _compute_product_cost_price(self, direction):
         """
         Update the cost price on the given product.
         The direction must be "in" if incoming and "out" if outgoing.
@@ -537,59 +549,38 @@ class Move(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Uom = pool.get('product.uom')
         Product = pool.get('product.product')
-        ProductTemplate = pool.get('product.template')
-        Location = pool.get('stock.location')
         Currency = pool.get('currency.currency')
-        Date = pool.get('ir.date')
 
         if direction == 'in':
             quantity = self.quantity
         elif direction == 'out':
             quantity = -self.quantity
-        context = {}
-        locations = Location.search([
-                ('type', '=', 'storage'),
-                ])
-        context['with_childs'] = False
-        context['locations'] = [l.id for l in locations]
-        context['stock_date_end'] = Date.today()
-        with Transaction().set_context(context):
-            product = Product(self.product.id)
-        qty = Uom.compute_qty(self.uom, quantity, product.default_uom)
+        qty = Uom.compute_qty(self.uom, quantity, self.product.default_uom)
 
         qty = Decimal(str(qty))
-        if not isinstance(Product.cost_price, TemplateFunction):
-            product_qty = product.quantity
-        else:
-            product_qty = product.template.quantity
-        product_qty = Decimal(str(max(product_qty, 0)))
+        product_qty = Decimal(str(self.product.quantity))
         # convert wrt currency
         with Transaction().set_context(date=self.effective_date):
             unit_price = Currency.compute(self.currency, self.unit_price,
                 self.company.currency, round=False)
         # convert wrt to the uom
         unit_price = Uom.compute_price(self.uom, unit_price,
-            product.default_uom)
-        if product_qty + qty != Decimal('0.0'):
+            self.product.default_uom)
+        cost_price = self.product.get_multivalue(
+            'cost_price', **self._cost_price_pattern)
+        if product_qty + qty > 0 and product_qty >= 0:
             new_cost_price = (
-                (product.cost_price * product_qty) + (unit_price * qty)
+                (cost_price * product_qty) + (unit_price * qty)
                 ) / (product_qty + qty)
-        else:
-            new_cost_price = product.cost_price
+        elif direction == 'in':
+            new_cost_price = unit_price
+        elif direction == 'out':
+            new_cost_price = cost_price
 
-        if not isinstance(Product.cost_price, TemplateFunction):
-            digits = Product.cost_price.digits
-            write = partial(Product.write, [product])
-        else:
-            digits = ProductTemplate.cost_price.digits
-            write = partial(ProductTemplate.write, [product.template])
-        new_cost_price = new_cost_price.quantize(
+        digits = Product.cost_price.digits
+        return new_cost_price.quantize(
             Decimal(str(10.0 ** -digits[1])))
 
-        write({
-                'cost_price': new_cost_price,
-                })
-
     @staticmethod
     def _get_internal_quantity(quantity, uom, product):
         Uom = Pool().get('product.uom')
@@ -624,26 +615,93 @@ class Move(Workflow, ModelSQL, ModelView):
     @ModelView.button
     @Workflow.transition('done')
     def do(cls, moves):
+        pool = Pool()
+        Product = pool.get('product.product')
+
+        def set_cost_values(cost_values):
+            Value = Product.multivalue_model('cost_price')
+            values = []
+            for product, cost_price, pattern in cost_values:
+                values.extend(product.set_multivalue(
+                        'cost_price', cost_price, save=False, **pattern))
+            Value.save(values)
+
         cls.check_origin(moves)
-        for move in moves:
-            move.set_effective_date()
-            move._do()
-            move.state = 'done'
-            # This save() call can't be grouped because the average computation
-            # of product cost price requires each move to be done separately
-            move.save()
+        for key, grouped_moves in groupby(moves, key=cls._cost_price_key):
+            to_save = []
+            cost_values = []
+            products = set()
+            grouped_moves = list(grouped_moves)
+            context = dict(key)
+            context.update(cls._cost_price_context(grouped_moves))
+            with Transaction().set_context(context):
+                grouped_moves = cls.browse(grouped_moves)
+                for move in grouped_moves:
+                    if move.product in products:
+                        # The average computation of product cost price
+                        # requires each previous move of the same product to be
+                        # saved
+                        cls.save(to_save)
+                        set_cost_values(cost_values)
+                        del to_save[:]
+                        del cost_values[:]
+                        products.clear()
+
+                    move.set_effective_date()
+                    cost_price = move._do()
+                    if cost_price is not None:
+                        cost_values.append(
+                            (move.product, cost_price,
+                                move._cost_price_pattern))
+                    if move.cost_price is None:
+                        if cost_price is None:
+                            cost_price = move.product.get_multivalue(
+                                'cost_price', **move._cost_price_pattern)
+                        move.cost_price = cost_price
+                    move.state = 'done'
+
+                    to_save.append(move)
+                    products.add(move.product)
+
+                if to_save:
+                    cls.save(to_save)
+                if cost_values:
+                    set_cost_values(cost_values)
+
+    @property
+    def _cost_price_pattern(self):
+        return {
+            'company': self.company.id,
+            }
+
+    def _cost_price_key(self):
+        return (
+            ('company', self.company.id),
+            )
+
+    @classmethod
+    def _cost_price_context(cls, moves):
+        pool = Pool()
+        Location = pool.get('stock.location')
+        Date = pool.get('ir.date')
+        context = {}
+        locations = Location.search([
+                ('type', '=', 'storage'),
+                ])
+        context['with_childs'] = False
+        context['locations'] = [l.id for l in locations]
+        context['stock_date_end'] = Date.today()
+        return context
 
     def _do(self):
         if (self.from_location.type in ('supplier', 'production')
                 and self.to_location.type == 'storage'
                 and self.product.cost_price_method == 'average'):
-            self._update_product_cost_price('in')
+            return self._compute_product_cost_price('in')
         elif (self.to_location.type == 'supplier'
                 and self.from_location.type == 'storage'
                 and self.product.cost_price_method == 'average'):
-            self._update_product_cost_price('out')
-        if self.cost_price is None:
-            self.cost_price = self.product.cost_price
+            return self._compute_product_cost_price('out')
 
     @classmethod
     @ModelView.button
@@ -708,8 +766,8 @@ class Move(Workflow, ModelSQL, ModelView):
     @classmethod
     def delete(cls, moves):
         for move in moves:
-            if move.state not in ('draft', 'cancel'):
-                cls.raise_user_error('del_draft_cancel', (move.rec_name,))
+            if move.state not in {'staging', 'draft', 'cancel'}:
+                cls.raise_user_error('delele_state', (move.rec_name,))
         super(Move, cls).delete(moves)
 
     @staticmethod
@@ -723,12 +781,19 @@ class Move(Workflow, ModelSQL, ModelView):
             types = cls.check_origin_types()
         if not types:
             return
-        for move in moves:
-            if ((move.from_location.type in types
-                        or move.to_location.type in types)
-                    and not move.origin):
-                cls.raise_user_warning('%s.done' % move,
-                    'no_origin', move.rec_name)
+
+        def no_origin(move):
+            return ((move.from_location.type in types) ^
+                (move.to_location.type in types)
+                and not move.origin)
+        moves = filter(no_origin, moves)
+        if moves:
+            names = ', '.join(m.rec_name for m in moves[:5])
+            if len(moves) > 5:
+                names += '...'
+            warning_name = '%s.done' % hashlib.md5(
+                str(moves).encode('utf-8')).hexdigest()
+            cls.raise_user_warning(warning_name, 'no_origin', names)
 
     def pick_product(self, location_quantities):
         """
@@ -952,6 +1017,84 @@ class Move(Workflow, ModelSQL, ModelView):
         if PeriodCache:
             period_cache = PeriodCache.__table__()
 
+        if with_childs:
+            # Replace tables with union which replaces flat children locations
+            # by their parent location.
+            from_location = Location.__table__()
+            from_parent_location = Location.__table__()
+            to_location = Location.__table__()
+            to_parent_location = Location.__table__()
+            columns = ['id', 'state', 'effective_date', 'planned_date',
+                'internal_quantity'] + list(grouping)
+            columns = [Column(move, c).as_(c) for c in columns]
+
+            move_with_parent = (move
+                .join(from_location,
+                    condition=move.from_location == from_location.id)
+                .join(from_parent_location, type_='LEFT',
+                    condition=from_location.parent == from_parent_location.id)
+                .join(to_location,
+                    condition=move.to_location == to_location.id)
+                .join(to_parent_location, type_='LEFT',
+                    condition=to_location.parent == to_parent_location.id))
+
+            move = Union(
+                # Moves not linked to any flat location
+                move_with_parent.select(
+                    move.from_location.as_('from_location'),
+                    move.to_location.as_('to_location'),
+                    *columns,
+                    where=(Coalesce(from_parent_location.flat_childs, False)
+                        != True)
+                    & (Coalesce(to_parent_location.flat_childs, False)
+                        != True)),
+                # Append moves linked from/to flat locations to their parents
+                move_with_parent.select(
+                    Case(
+                        (from_parent_location.flat_childs,
+                            from_parent_location.id),
+                        else_=move.from_location).as_('from_location'),
+                    Case(
+                        (to_parent_location.flat_childs,
+                            to_parent_location.id),
+                        else_=move.to_location).as_('to_location'),
+                    *columns,
+                    where=(from_parent_location.flat_childs == True)
+                    | (to_parent_location.flat_childs == True)),
+                # Append moves linked to from/to flat locations only
+                move_with_parent.select(
+                    Case(
+                        (from_parent_location.flat_childs,
+                            from_location.id),
+                        else_=Null).as_('from_location'),
+                    Case(
+                        (to_parent_location.flat_childs,
+                            to_location.id),
+                        else_=Null).as_('to_location'),
+                    *columns,
+                    where=(from_parent_location.flat_childs == True)
+                    | (to_parent_location.flat_childs == True)),
+                all_=True)
+
+            if PeriodCache:
+                location = Location.__table__()
+                parent_location = Location.__table__()
+                columns = ['internal_quantity', 'period'] + list(grouping)
+                columns = [Column(period_cache, c).as_(c) for c in columns]
+                period_cache = Union(
+                    period_cache.select(
+                        period_cache.location.as_('location'),
+                        *columns),
+                    period_cache.join(location,
+                        condition=period_cache.location == location.id
+                        ).join(parent_location, type_='LEFT',
+                        condition=location.parent == parent_location.id
+                        ).select(
+                        parent_location.id.as_('location'),
+                        *columns,
+                        where=parent_location.flat_childs == True),
+                    all_=True)
+
         if not context.get('stock_date_end'):
             context['stock_date_end'] = datetime.date.max
 
@@ -1092,9 +1235,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 state_date_clause_out &= state_date_clause()
 
         if with_childs:
-            location_query = Location.search([
-                    ('parent', 'child_of', location_ids),
-                    ], query=True, order=[])
+            location_query = _location_children(location_ids, query=True)
         else:
             location_query = location_ids[:]
 
@@ -1191,7 +1332,6 @@ class Move(Workflow, ModelSQL, ModelView):
             and quantity as value.
         """
         pool = Pool()
-        Location = pool.get('stock.location')
         Product = pool.get('product.product')
 
         assert query is not None, (
@@ -1223,15 +1363,13 @@ class Move(Workflow, ModelSQL, ModelView):
         # Propagate quantities on from child locations to their parents
         if with_childs and len(location_ids) > 1:
             # Fetch all child locations
-            locations = Location.search([
-                    ('parent', 'child_of', location_ids),
-                    ])
+            locations = _location_children(location_ids)
             # Generate a set of locations without childs and a dict
             # giving the parent of each location.
             leafs = set([l.id for l in locations])
             parent = {}
             for location in locations:
-                if not location.parent:
+                if not location.parent or location.parent.flat_childs:
                     continue
                 if location.parent.id in leafs:
                     leafs.remove(location.parent.id)
@@ -1270,3 +1408,26 @@ class Move(Workflow, ModelSQL, ModelView):
             quantities[key] = uom.round(quantity)
 
         return quantities
+
+
+def _location_children(location_ids, query=False):
+    "Return children location without including flat children"
+    pool = Pool()
+    Location = pool.get('stock.location')
+    nested_location_ids = []
+    flat_location_ids = []
+    for location in Location.browse(location_ids):
+        if location.flat_childs:
+            flat_location_ids.append(location.id)
+        else:
+            nested_location_ids.append(location.id)
+    if nested_location_ids:
+        return Location.search(['OR',
+                ('parent', 'child_of', nested_location_ids),
+                ('id', 'in', flat_location_ids),
+                ], query=query, order=[])
+    else:
+        if query:
+            return flat_location_ids
+        else:
+            return Location.browse(flat_location_ids)
