@@ -140,8 +140,10 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
             'Inventory Moves',
             domain=[
                 ('from_location', '=', Eval('warehouse_input')),
-                ('to_location', 'child_of', [Eval('warehouse_storage', -1)],
-                    'parent'),
+                If(~Eval('state').in_(['done', 'cancel']),
+                    ('to_location', 'child_of',
+                        [Eval('warehouse_storage', -1)], 'parent'),
+                    (),),
                 ('company', '=', Eval('company')),
                 ],
             states={
@@ -577,12 +579,14 @@ class ShipmentInReturn(Workflow, ModelSQL, ModelView):
             If(Eval('state') == 'draft', [
                     ('from_location', '=', Eval('from_location')),
                     ('to_location', '=', Eval('to_location')),
-                    ], [
-                    ('from_location', 'child_of', [Eval('from_location', -1)],
-                        'parent'),
-                    ('to_location', 'child_of', [Eval('to_location', -1)],
-                        'parent'),
-                    ]),
+                    ],
+                If(~Eval('state').in_(['done', 'cancel']), [
+                        ('from_location', 'child_of',
+                            [Eval('from_location', -1)], 'parent'),
+                        ('to_location', 'child_of',
+                            [Eval('to_location', -1)], 'parent'),
+                        ],
+                    [])),
             ('company', '=', Eval('company')),
             ],
         depends=['state', 'from_location', 'to_location', 'company'])
@@ -833,8 +837,9 @@ class ShipmentInReturn(Workflow, ModelSQL, ModelView):
     def assign_try(cls, shipments):
         pool = Pool()
         Move = pool.get('stock.move')
-        if Move.assign_try([m for s in shipments for m in s.moves],
-                with_childs=False):
+        to_assign = [m for s in shipments for m in s.moves
+            if m.assignation_required]
+        if Move.assign_try(to_assign, with_childs=False):
             cls.assign(shipments)
             return True
         else:
@@ -914,8 +919,10 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
     inventory_moves = fields.Function(fields.One2Many('stock.move', 'shipment',
             'Inventory Moves',
             domain=[
-                ('from_location', 'child_of', [Eval('warehouse_storage', -1)],
-                    'parent'),
+                If(Eval('state').in_(['waiting']),
+                    ('from_location', 'child_of',
+                        [Eval('warehouse_storage', -1)], 'parent'),
+                    ()),
                 ('to_location', '=', Eval('warehouse_output')),
                 ('company', '=', Eval('company')),
                 ],
@@ -1425,8 +1432,9 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
     @ModelView.button
     def assign_try(cls, shipments):
         Move = Pool().get('stock.move')
-        if Move.assign_try([m for s in shipments
-                    for m in s.inventory_moves]):
+        to_assign = [m for s in shipments for m in s.inventory_moves
+            if m.assignation_required]
+        if Move.assign_try(to_assign):
             cls.assign(shipments)
             return True
         else:
@@ -1508,8 +1516,10 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
             'Inventory Moves',
             domain=[
                 ('from_location', '=', Eval('warehouse_input')),
-                ('to_location', 'child_of', [Eval('warehouse_storage', -1)],
-                    'parent'),
+                If(Eval('state').in_(['received']),
+                    ('to_location', 'child_of',
+                        [Eval('warehouse_storage', -1)], 'parent'),
+                    ()),
                 ('company', '=', Eval('company')),
                 ],
             states={
@@ -1956,25 +1966,28 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
                     ('from_location', '=', Eval('from_location')),
                     ('to_location', '=', Eval('to_location')),
                     ],
-                If(~Eval('transit_location'),
-                    [
-                        ('from_location', 'child_of',
-                            [Eval('from_location', -1)], 'parent'),
-                        ('to_location', 'child_of',
-                            [Eval('to_location', -1)], 'parent'),
-                        ],
-                    ['OR',
+                If(~Eval('state').in_(['done', 'cancel']),
+                    If(~Eval('transit_location'),
                         [
                             ('from_location', 'child_of',
                                 [Eval('from_location', -1)], 'parent'),
-                            ('to_location', '=', Eval('transit_location')),
-                            ],
-                        [
-                            ('from_location', '=', Eval('transit_location')),
                             ('to_location', 'child_of',
                                 [Eval('to_location', -1)], 'parent'),
                             ],
-                        ])),
+                        ['OR',
+                            [
+                                ('from_location', 'child_of',
+                                    [Eval('from_location', -1)], 'parent'),
+                                ('to_location', '=', Eval('transit_location')),
+                                ],
+                            [
+                                ('from_location', '=',
+                                    Eval('transit_location')),
+                                ('to_location', 'child_of',
+                                    [Eval('to_location', -1)], 'parent'),
+                                ],
+                            ]),
+                    [])),
             ('company', '=', Eval('company')),
             ],
         depends=['state', 'from_location', 'to_location', 'transit_location',
@@ -1982,12 +1995,15 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
     outgoing_moves = fields.Function(fields.One2Many('stock.move', 'shipment',
             'Outgoing Moves',
             domain=[
-                ('from_location', 'child_of', [Eval('from_location', -1)],
-                    'parent'),
-                If(~Eval('transit_location'),
-                    ('to_location', 'child_of', [Eval('to_location', -1)],
-                        'parent'),
-                    ('to_location', '=', Eval('transit_location'))),
+                If(Eval('state').in_(['request', 'draft']), [
+                        ('from_location', 'child_of',
+                            [Eval('from_location', -1)], 'parent'),
+                        If(~Eval('transit_location'),
+                            ('to_location', 'child_of',
+                                [Eval('to_location', -1)], 'parent'),
+                            ('to_location', '=', Eval('transit_location'))),
+                        ],
+                    []),
                 ],
             states={
                 'readonly': Eval('state').in_(
@@ -2001,12 +2017,15 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
     incoming_moves = fields.Function(fields.One2Many('stock.move', 'shipment',
             'Incoming Moves',
             domain=[
-                If(~Eval('transit_location'),
-                    ('from_location', 'child_of', [Eval('from_location', -1)],
-                        'parent'),
-                    ('from_location', '=', Eval('transit_location'))),
-                ('to_location', 'child_of', [Eval('to_location', -1)],
-                    'parent'),
+                If(~Eval('state').in_(['done', 'cancel']), [
+                        If(~Eval('transit_location'),
+                            ('from_location', 'child_of',
+                                [Eval('from_location', -1)], 'parent'),
+                            ('from_location', '=', Eval('transit_location'))),
+                        ('to_location', 'child_of',
+                            [Eval('to_location', -1)], 'parent'),
+                        ],
+                    []),
                 ],
             states={
                 'readonly': Eval('state').in_(['done', 'cancel']),
@@ -2375,6 +2394,7 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
         pass
 
     @classmethod
+    @ModelView.button
     @Workflow.transition('shipped')
     @set_employee('shipped_by')
     def ship(cls, shipments):
@@ -2417,7 +2437,7 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
     def assign_try(cls, shipments):
         Move = Pool().get('stock.move')
         to_assign = [m for s in shipments for m in s.outgoing_moves
-            if m.from_location.type != 'lost_found']
+            if m.assignation_required]
         if not to_assign or Move.assign_try(to_assign):
             cls.assign(shipments)
             return True
