@@ -13,7 +13,7 @@ from trytond.tests.test_tryton import ModuleTestCase, with_transaction
 from trytond.tests.test_tryton import doctest_teardown
 from trytond.tests.test_tryton import doctest_checker
 from trytond.transaction import Transaction
-from trytond.exceptions import UserWarning
+from trytond.exceptions import UserWarning, UserError
 from trytond.pool import Pool
 
 from trytond.modules.company.tests import create_company, set_company
@@ -40,8 +40,6 @@ class StockTestCase(ModuleTestCase):
                     'name': 'Test Move.internal_quantity',
                     'type': 'goods',
                     'list_price': Decimal(1),
-                    'cost_price': Decimal(0),
-                    'cost_price_method': 'fixed',
                     'default_uom': kg.id,
                     }])
         product, = Product.create([{
@@ -98,8 +96,6 @@ class StockTestCase(ModuleTestCase):
                     'name': 'Test products_by_location',
                     'type': 'goods',
                     'list_price': Decimal(0),
-                    'cost_price': Decimal(0),
-                    'cost_price_method': 'fixed',
                     'default_uom': kg.id,
                     }])
         product, = Product.create([{
@@ -364,8 +360,6 @@ class StockTestCase(ModuleTestCase):
                     'name': 'Test products_by_location',
                     'type': 'goods',
                     'list_price': Decimal(0),
-                    'cost_price': Decimal(0),
-                    'cost_price_method': 'fixed',
                     'default_uom': unit.id,
                     }])
         product, = Product.create([{
@@ -426,15 +420,136 @@ class StockTestCase(ModuleTestCase):
                 products_by_location[(warehouse.id, product.id)], 1)
             self.assertEqual(products_by_location[(storage.id, product.id)], 2)
 
+            with Transaction().set_context(locations=[warehouse.id]):
+                found_products = Product.search([
+                        ('quantity', '=', 1),
+                        ])
+                self.assertListEqual([product], found_products)
+
             with Transaction().set_context(stock_skip_warehouse=True):
                 products_by_location = Product.products_by_location(
                     [warehouse.id], [product.id], with_childs=True)
                 products_by_location_all = Product.products_by_location(
                     [warehouse.id], None, with_childs=True)
+                self.assertEqual(
+                    products_by_location[(warehouse.id, product.id)], 2)
+                self.assertEqual(
+                    products_by_location_all[(warehouse.id, product.id)], 2)
+
+                with Transaction().set_context(locations=[warehouse.id]):
+                    found_products = Product.search([
+                            ('quantity', '=', 2),
+                            ])
+                    self.assertListEqual([product], found_products)
+
+    @with_transaction()
+    def test_products_by_location_flat_childs(self, period_closed=False):
+        "Test products_by_location on flat_childs"
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+        Date = pool.get('ir.date')
+        Period = pool.get('stock.period')
+
+        unit, = Uom.search([('name', '=', 'Unit')])
+        template, = Template.create([{
+                    'name': "Product",
+                    'type': 'goods',
+                    'list_price': Decimal(0),
+                    'default_uom': unit.id,
+                    }])
+        product, = Product.create([{
+                    'template': template.id,
+                    }])
+
+        lost_found, = Location.search([('type', '=', 'lost_found')])
+        storage, = Location.search([('code', '=', 'STO')])
+        storage.flat_childs = True
+        storage.save()
+        storage1, = Location.create([{
+                    'name': 'Storage 1',
+                    'type': 'storage',
+                    'parent': storage.id,
+                    }])
+        storage2, = Location.create([{
+                    'name': 'Storage 2',
+                    'type': 'storage',
+                    'parent': storage.id,
+                    }])
+
+        company = create_company()
+        with set_company(company):
+            date = Date.today() - relativedelta(days=1)
+
+            moves = Move.create([{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 1,
+                        'from_location': lost_found.id,
+                        'to_location': storage.id,
+                        'planned_date': date,
+                        'effective_date': date,
+                        'company': company.id,
+                        }, {
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 1,
+                        'from_location': lost_found.id,
+                        'to_location': storage1.id,
+                        'planned_date': date,
+                        'effective_date': date,
+                        'company': company.id,
+                        }, {
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 1,
+                        'from_location': storage1.id,
+                        'to_location': storage2.id,
+                        'planned_date': date,
+                        'effective_date': date,
+                        'company': company.id,
+                        }])
+            Move.do(moves)
+
+            if period_closed:
+                period, = Period.create([{
+                            'date': date,
+                            'company': company.id,
+                            }])
+                Period.close([period])
+
+            # Test flat location
+            products_by_location = Product.products_by_location(
+                [storage.id], [product.id], with_childs=True)
             self.assertEqual(
-                products_by_location[(warehouse.id, product.id)], 2)
+                products_by_location[(storage.id, product.id)], 2)
+
+            # Test mixed flat and nested location
+            products_by_location = Product.products_by_location(
+                [storage.parent.id, storage.id, storage1.id, storage2.id],
+                [product.id],
+                with_childs=True)
             self.assertEqual(
-                products_by_location_all[(warehouse.id, product.id)], 2)
+                products_by_location[(storage.parent.id, product.id)], 2)
+            self.assertEqual(
+                products_by_location[(storage.id, product.id)], 2)
+            self.assertEqual(
+                products_by_location[(storage1.id, product.id)], 0)
+            self.assertEqual(
+                products_by_location[(storage2.id, product.id)], 1)
+
+            # Test non flat
+            products_by_location = Product.products_by_location(
+                [lost_found.id], [product.id], with_childs=True)
+            self.assertEqual(
+                products_by_location[(lost_found.id, product.id)], -2)
+
+    def test_products_by_location_flat_childs_period_closed(self):
+        "Test products_by_location on flat_childs with period closed"
+        self.test_products_by_location_flat_childs(period_closed=True)
 
     @with_transaction()
     def test_period(self):
@@ -452,10 +567,8 @@ class StockTestCase(ModuleTestCase):
         template, = Template.create([{
                     'name': 'Test period',
                     'type': 'goods',
-                    'cost_price_method': 'fixed',
                     'default_uom': unit.id,
                     'list_price': Decimal(0),
-                    'cost_price': Decimal(0),
                     }])
         product, = Product.create([{
                     'template': template.id,
@@ -624,8 +737,6 @@ class StockTestCase(ModuleTestCase):
                     'name': 'Test Move.check_origin',
                     'type': 'goods',
                     'list_price': Decimal(1),
-                    'cost_price': Decimal(0),
-                    'cost_price_method': 'fixed',
                     'default_uom': uom.id,
                     }])
         product, = Product.create([{
@@ -676,8 +787,6 @@ class StockTestCase(ModuleTestCase):
             name='Test Move.assign_try',
             type='goods',
             list_price=Decimal(1),
-            cost_price=Decimal(0),
-            cost_price_method='fixed',
             default_uom=uom,
             )
         template.save()
@@ -743,8 +852,6 @@ class StockTestCase(ModuleTestCase):
             name='Test Move.assign_try',
             type='goods',
             list_price=Decimal(1),
-            cost_price=Decimal(0),
-            cost_price_method='fixed',
             default_uom=uom,
             )
         template.save()
@@ -810,8 +917,6 @@ class StockTestCase(ModuleTestCase):
             name="Product",
             type='goods',
             list_price=Decimal(1),
-            cost_price=Decimal(1),
-            cost_price_method='fixed',
             default_uom=unit,
             )
         template.save()
@@ -887,6 +992,107 @@ class StockTestCase(ModuleTestCase):
                 pbl = Product.products_by_location([storage.id], [product.id])
                 self.assertDictEqual(pbl, {(storage.id, product.id): -5})
 
+    @with_transaction()
+    def test_location_inactive_without_move(self):
+        "Test inactivate location without move"
+        pool = Pool()
+        Location = pool.get('stock.location')
+        storage, = Location.search([('code', '=', 'STO')])
+        location, = Location.create([{
+                    'name': "Location",
+                    'parent': storage.id,
+                    }])
+
+        location.active = False
+        location.save()
+
+    @with_transaction()
+    def test_location_inactive_with_quantity(self):
+        "Test inactivate location with quantity"
+        pool = Pool()
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+
+        storage, = Location.search([('code', '=', 'STO')])
+        location, = Location.create([{
+                    'name': "Location",
+                    'parent': storage.id,
+                    }])
+        unit, = Uom.search([('name', '=', "Unit")])
+        template, = Template.create([{
+                    'name': "Product",
+                    'type': 'goods',
+                    'list_price': Decimal(0),
+                    'cost_price': Decimal(0),
+                    'default_uom': unit.id,
+                    }])
+        product, = Product.create([{'template': template.id}])
+
+        company = create_company()
+        with set_company(company):
+
+            moves = Move.create([{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 1,
+                        'from_location': storage.id,
+                        'to_location': location.id,
+                        'company': company.id,
+                        }])
+            Move.do(moves)
+            with self.assertRaises(UserError):
+                location.active = False
+                location.save()
+
+    @with_transaction()
+    def test_location_inactive_with_draft_moves(self):
+        "Test inactivate location with draft moves"
+        pool = Pool()
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+
+        storage, = Location.search([('code', '=', 'STO')])
+        location, = Location.create([{
+                    'name': "Location",
+                    'parent': storage.id,
+                    }])
+        unit, = Uom.search([('name', '=', "Unit")])
+        template, = Template.create([{
+                    'name': "Product",
+                    'type': 'goods',
+                    'list_price': Decimal(0),
+                    'cost_price': Decimal(0),
+                    'default_uom': unit.id,
+                    }])
+        product, = Product.create([{'template': template.id}])
+
+        company = create_company()
+        with set_company(company):
+            Move.create([{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 1,
+                        'from_location': storage.id,
+                        'to_location': location.id,
+                        'company': company.id,
+                        }, {
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 1,
+                        'from_location': location.id,
+                        'to_location': storage.id,
+                        'company': company.id,
+                        }])
+            with self.assertRaises(UserError):
+                location.active = False
+                location.save()
+
 
 def suite():
     suite = trytond.tests.test_tryton.suite()
